/*
Copyright (c) 2007, Caridy Patio. All rights reserved.
Portions Copyright (c) 2007, Yahoo!, Inc. All rights reserved.
Code licensed under the BSD License:
http://www.bubbling-library.com/eng/licence
version: 1.3.2
*/
(function() {
  var $L = YAHOO.lang,
	  $C = YAHOO.util.Connect,
	  $E = YAHOO.util.Event,
	  $D = YAHOO.util.Dom,
	  $ =  YAHOO.util.Dom.get;

	/**
	* @singleton WizardManager - Use this object to management your wizards...
	* Apply dynamic functionality to a wizard widget
	* @constructor
	*/
    YAHOO.util.WizardManager = function() {
		var obj = {},
			_areas = {},
			_className = 'yui-cms-wizard',
			_loadingClass = 'loading',
			_form_control = 'yuicmswizard',
			_defConf   = {
							status: false,
							handle: null,
							uri: null,
							morePostDate: {},
							onCancel: null,
							onReady: null,
							onFinish: null,
							onSubmit: null,
							compileButton: null,
							values: {}
						 };

		var callback = {
			success: function(o) {
				var area = o.argument.area;
				// process the response here & compiling the result using the dispatcher...
				YAHOO.util.Dispatcher.process ( area.element, o.responseText, {
					before: function(){
					},
					error: function() {
								onError ( area );
						   },
					after: function(){
						// process the area
						$D.removeClass(area.element, _loadingClass);
						// looking for important moments in the execution thread
						if (area.values._wizardfinish) {
							onFinish(area);
						}
						else if (area.values_wizardcancel) {
							onCancel(area);
						}
						check ( area );
					}
				});

			},
			failure: function(o) {
				onError ( o.argument.area );
			}
		};
		function fetch( area, m, uri, c) {
			// set the loading class to the area...
			$D.addClass(area.element, _loadingClass);
			area.handle = $C.asyncRequest(m, uri, c);
		}

		function check (area) {
			var items = [], sName, k, bt, label, f;
			if ($L.isObject(area) && $L.isObject(area.element)) {
			    if (!area.overheat) {
				  // analysing the content of the area...
			  	  // searching submit buttons
				  items.merge(area.element.getElementsByTagName('input'));
	              // Loop through each result
	              for(k = 0; k < items.length; k++) {
					bt = items[k].getAttribute('type').toLowerCase();
	                label = items[k].getAttribute('value');
					if ($L.isObject(YAHOO.widget.Button) && ((bt == 'submit') || (bt == 'reset') || (bt == 'button'))) {
	                    // YUI buttons are incompletes, that's why I introduce a new level with class="i" to include icons in radios and checkbox
		                label = '<div class="i">'+label+'</div>';
	                    // Create a Button using an existing <input> element as a data source
						bt = new YAHOO.widget.Button(items[k], {label: label});
	                    if ($L.isFunction(area.compileButton)) {
							area.compileButton.apply ( area, [items[k], bt] );
						}
					}
				  }
				}
			  	// searching forms
				items = [];
				items.merge(area.element.getElementsByTagName('form'));
	            // Loop through each result
	            for(k = 0; k < items.length; k++) {
					f = items[k];
    				area.morePostData = area.morePostData || {};
    				area.morePostData[_form_control] = area.id;
					var cc = {
						success: callback.success,
						failure: callback.failure,
						argument: { area:area }
					};
					// applying the correct event for each form...
					$E.on(f, "submit", function (e) {
                        var sName = null, uri = f.getAttribute("action",2) || area.uri,
    						m = (f.getAttribute('method').toLowerCase() == 'post'?'POST':'GET');
    					// preparing the form...
    					$C._formNode = f;
    					for (sName in area.morePostData) {
    						if (area.morePostData.hasOwnProperty(sName)) {
    							$C.appendPostData (sName+'='+area.morePostData[sName]);
    						}
    					}
						onSubmit (area);
						// hacking the YUI Button Submit - To include the submit button values (deprecated after the 2.3.1)
						if ($L.isObject(YAHOO.widget.Button)) {
                          YAHOO.widget.Button.addHiddenFieldsToForm(f);
                        }
						$C.setForm(f);
						fetch (area, m, uri, cc);
        				// Hack: clear the YUI Connection Manager...
        				// if we leave the form linked to the connection manager object, all the futher call will have all this values...
        				$C.resetFormState();
					    $E.stopEvent(e);
					});
				}
				// everything was fine here...
				area.status = true;
	            onReady(area);
			}
		}
		function reset (area, uri) {
			uri = new String($L.isString(uri)?uri:area.uri);
			uri += ((uri.indexOf('?') == -1)?'?':'&')+_form_control+'='+area.id;
			if ($L.isObject(area) && $L.isObject(area.element = $(area.id)) && uri) {
			  // fetching the remote uri...
			  var cc = {
				success: callback.success,
				failure: callback.failure,
				argument: { area:area }
			  };
			  fetch (area, 'GET', uri, cc);
			}
		}

	    var onSubmit = function ( area ) {
			if ($L.isFunction(area.onSubmit)) {
				area.onSubmit.apply ( area, [area.values] );
			}
	    };
	    var onFinish = function ( area ) {
			if ($L.isFunction(area.onFinish)) {
				area.onFinish.apply ( area, [area.values] );
			}
	    };
		var onCancel = function ( area ) {
			if ($L.isFunction(area.onCancel)) {
				area.onCancel.apply ( area, [area.values] );
			}
		};
		var onError = function ( area ) {
			if ($L.isFunction(area.onError)) {
				area.onError.apply ( area, [area.values] );
			}
		};
		var onReady = function ( area ) {
			if ($L.isFunction(area.onReady)) {
				area.onReady.apply ( area, [area.values] );
			}
		};

		// public vars
		// public methods
		/**
		* * get the current status of the wizard
		* @public
		* @param {string} id   area Id
		* @return string
		*/
		obj.getStatus = function ( id ) {
			if ($L.isObject(_areas[id])) {
				return _areas[id].status;
			}
			return false;
		};
		/**
		* * get the values for the current wizard
		* @public
		* @param {string} id   area Id
		* @return boolean
		*/
		obj.getValues = function ( id ) {
			if (this.getStatus(id)) {
				return _areas[id].values;
			}
			return false;
		};
		/**
		* * set the values for the current wizard
		* @public
		* @param {string} id   area Id
		* @param {string} enc  json encoding string
		* @return Object
		*/
		obj.setValues = function ( id, enc ) {
			if (this.getStatus(id)) {
				var area = _areas[id];
				area.values = enc.parseJSON();
				return area.values;
			}
			return null;
		};

		/**
		* * use the jump method to manually jump to new uri
		* @public
		* @param {string} id   area Id
		* @param {string} uri  url for the next step
		* @return boolean
		*/
		obj.jump = function ( id, uri ) {
			if (this.getStatus(id)) {
				reset( _areas[id], uri );
				return true;
			}
			return false;
		};
		/**
		* * add a new area to the list
		* @public
		* @param {string} id
		* @return Object
		*/
		obj.add = function ( id, userConfig ) {
			var c = userConfig || _defConf,
				el = c.id || id ||  c.element;
			if ($L.isString(el) && (el !== '')) {
				el = $(el);
			}
			if ($L.isObject(el) && $L.isString(c.uri) && (c.uri !== '')) {
				c.id = $E.generateId(el);
				c.element = el;
				c.handle = null;
				c.status = false;
				c.values = {};
				this.remove(c.id);
				$D.addClass(c.id, _className);
				_areas[id] = c;
				reset ( _areas[id] );
			}
			return _areas[id];
		};
		/**
		* * adopt an area with the content inside and add to the list
		* @public
		* @param {string} id
		* @return Object
		*/
		obj.adopt = function ( id, userConfig ) {
			var c = userConfig || _defConf,
				el = c.id || id ||  c.element;
			if ($L.isString(el) && (el !== '')) {
				el = $(el);
			}
			if ($L.isObject(el)) {
				c.id = $E.generateId(el);
				c.element = el;
				c.handle = null;
				c.status = false;
				c.values = {};
				this.remove(c.id);
				$D.addClass(c.id, _className);
				_areas[id] = c;
				// process the current area's content...
				check ( _areas[id] );
			}
			return _areas[id];
		};
		/**
		* * Remove an area from the stock...
		* @public
		* @param {object} id	className
		* @return void
		*/
		obj.remove = function ( id ) {
			if (id && (_areas[id])) {
				_areas[id].handle = null; // resetting the handle
				_areas[id] = null; // discarding the area...
			}
			_areas[id] = [];
		};
		/**
		* * Reload an area...
		* @public
		* @param {object} id	className
		* @return void
		*/
		obj.reload = function ( id ) {
			if (id && (_areas[id])) {
				reset (_areas[id]);
			}
		};
		return obj;
    }();

})();
/* Array prototype, marge a new array to the current array */
Array.prototype.merge = function ( items ) {
	var i;
	for (i=0; i < items.length; i++) {
      this.push(items[i]);
	}
};
YAHOO.register("wizard", YAHOO.util.WizardManager, {version: "1.3.2", build: "206"});